#include "stdafx.h"
#include "resource.h"		// main symbols
#include "StickyDragManager.h"

BOOL CALLBACK enumWindows(
  HWND hwnd,      // handle to window
  LPARAM lParam   // application-defined value
)
{
	CStickyDragManager* smgr = (CStickyDragManager*)lParam;
	if (hwnd != smgr->GetActiveWindow())
		smgr->AddWindow(hwnd);
	return TRUE;
}

class CStickyWindow : public CWnd
{
public:
	CStickyWindow()
	{
	}
	void SetDocker(bool isDocker) { m_dragManager.SetDockerState(isDocker); }
protected:
	CStickyDragManager m_dragManager;
protected:
	// Generated message map functions
	//{{AFX_MSG(CStickyWindow)
	afx_msg void OnNcLButtonDown(UINT nHitTest, CPoint point)
	{
		m_dragManager.Init(GetSafeHwnd());
		// Register application windows.
		EnumThreadWindows(GetCurrentThreadId(), enumWindows, (LPARAM)&m_dragManager);
		// Throw in the desktop window for good measure.
		m_dragManager.AddDesktopWindow();
		m_dragManager.StartTrack();
		RECT r;
		GetWindowRect(&r);
		
		m_relativeOffset.x = point.x - r.left;
		m_relativeOffset.y = point.y - r.top;

		CWnd::OnNcLButtonDown(nHitTest, point);
	}
	afx_msg void OnSizing(UINT fwSide, LPRECT pRect)
	{
		m_dragManager.SnapSize(pRect);
	}
	afx_msg void OnMoving(UINT fwSide, LPRECT pRect)
	{
		POINT pt;
		GetCursorPos(&pt);
		vector<RECT> rects;
		RECT saveR = *pRect;
		vector<HWND> *windows = m_dragManager.SnapMove(pRect, rects);
		//Correct for original mouse position if no snap behavior occurs...
		if (EqualRect(pRect, &saveR))
		{
			int dx = pt.x - pRect->left;
			int dy = pt.y - pRect->top;
			if ((abs(dx-m_relativeOffset.x) > 5) || (abs(dy-m_relativeOffset.y) > 5))
				OffsetRect(pRect, pt.x-saveR.left-m_relativeOffset.x, pt.y-saveR.top-m_relativeOffset.y);
		}

		// Note - this will only look right in show-contents-while-dragging mode.
		// if you need better performance or OS aggreement - you'll have to write your own dragger.
		HDWP hDWP = BeginDeferWindowPos(windows->size());
		for (unsigned int i = 0; i < windows->size(); i++)
			::DeferWindowPos(hDWP, (*windows)[i], 0, rects[i].left, rects[i].top, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);
//			::SetWindowPos((*windows)[i], 0, rects[i].left, rects[i].top, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER);
		EndDeferWindowPos(hDWP);
	}
	afx_msg void OnClose()
	{
		CWnd::OnClose();
		AfxPostQuitMessage(0);
	}
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
	CPoint m_relativeOffset;
};

BEGIN_MESSAGE_MAP(CStickyWindow, CWnd)
	//{{AFX_MSG_MAP(CStickyWindow)
	ON_WM_NCLBUTTONDOWN()
	ON_WM_SIZING()
	ON_WM_MOVING()
	ON_WM_CLOSE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


class CStickyWindowsApp : public CWinApp
{
public:
	CStickyWindowsApp()
	{
	}
	virtual BOOL InitInstance()
	{
		CRect pr(20, 20, 220, 220);
		for (int i = 0; i < 8; i++, pr += CPoint(20, 20))
		{
			CString title;
			if (i == 2 || i == 5)
				title.Format("Sticky Window Docker -%d-", i);
			else
				title.Format("Sticky Window -%d-", i);
			m_windows[i].CreateEx(WS_EX_TOOLWINDOW, AfxRegisterWndClass(CS_VREDRAW | CS_HREDRAW, 0, GetSysColorBrush(COLOR_BTNFACE), 0),
				title, WS_OVERLAPPED | WS_SYSMENU | WS_THICKFRAME, pr, CWnd::GetDesktopWindow(), 0);
			if (i == 2 || i == 5)
				m_windows[i].SetDocker(true);
			m_windows[i].ShowWindow(SW_SHOW);
		}
		m_pMainWnd = &m_windows[0];
		return TRUE;
	}
	CStickyWindow m_windows[8];
};

CStickyWindowsApp theApp;

